﻿module sslsocketstream;

pragma(lib, "libeay32.lib");
pragma(lib, "ssleay32.lib");

import openssl;

string SSLgeterror() {
	extern(C) static int set_buf(char* str, size_t len, void* p) {
		auto f = cast(const(char)[]*)p;
		*f = str[0..len].idup;
		return len;
	}

	string str;
	ERR_print_errors_cb(&set_buf, &str);
	return str;
}

import std.contracts : enforce;
import std.socketstream, std.socket, std.stream;
class SSLSocketStream : SocketStream {
	static SSL_CTX* ctx;
	static bool inited;
	SSL* ssl;

	this(Socket sock) {
		super(sock);

		if (!inited) {
			SSL_load_error_strings();
			SSL_library_init();
			ctx = enforce(SSL_CTX_new(SSLv23_client_method()), SSLgeterror());
		}

		ssl = enforce(SSL_new(ctx), SSLgeterror());
		enforce(SSL_set_fd(ssl, socket.handle), SSLgeterror());

		RAND_poll();
		while (RAND_status() == 0){
			ushort rand_ret = rand() & 0xFFFF;
			RAND_seed(&rand_ret, rand_ret.sizeof);
		}

		enforce(SSL_connect(ssl) > 0, SSLgeterror());
	}

	static ~this() {
		if (inited) {
			SSL_CTX_free(ctx);
			ERR_free_strings();
		}
	}

	override size_t readBlock(void* buffer, size_t size) {
		assertReadable();

		if (size == 0)
			return size;

		auto len = SSL_read(ssl, buffer, size);
		readEOF = cast(bool)(len == 0);
		if (len <= 0)
			len = 0;

		return len;
	}

	override size_t writeBlock(const void* buffer, size_t size) {
		assertWriteable();

		if (size == 0)
			return size;

		auto len = SSL_write(ssl, buffer, size);
		//readEOF = cast(bool)(len == 0);
		if (len <= 0)
			len = 0;

		return len;
	}

	override ulong seek(long offset, SeekPos whence)
	{
		throw new SeekException("Cannot seek a socket.");
	}

	override void close() {
		enforce(SSL_shutdown(ssl) == 1, SSLgeterror()); 
		SSL_free(ssl); 
		SocketStream.close();
	}
}
